#include "PiecemealFeed.h"

#include <Wt/Http/Response>
#include <Wt/Utils>
#include <Wt/WApplication>

#include <chrono>

void timeout(PiecemealFeed * feed)
{
    while (feed->isRunning())
    {
        std::this_thread::sleep_for(std::chrono::seconds{ 1 });
    }
}

PiecemealFeed::PiecemealFeed()
{}

PiecemealFeed::~PiecemealFeed()
{
    beingDeleted();
}

bool PiecemealFeed::isRunning()
{
    if (running)
    {
        haveMoreData();
    }
    std::unique_lock<std::mutex> guard(lock);
    return running;
}

void PiecemealFeed::handleRequest(const Wt::Http::Request &request,
    Wt::Http::Response &response)
{
    std::unique_lock<std::mutex> guard(lock);

    int count = 1;

    if (auto * prev = request.continuation())
    {
        count = boost::any_cast<int>(prev->data());
    }
    else
    {
        timer = std::thread{ timeout, this };
    }

    response.setMimeType("application/json");

    response.out() << "Current count is " << count << "\n";

    std::cerr << "Handling count " << count << "\n";

    auto next = response.createContinuation()->shared_from_this();
    next->setData(count + 1);
    next->waitForMoreData();
}

void PiecemealFeed::handleAbort(const Wt::Http::Request & request)
{
    std::unique_lock<std::mutex> guard(lock);

    std::cerr << "Handling abort\n";

    running = false;
}
